@@ -1,7 +1,7 @@
 import logging
 import multiprocessing
 from concurrent import futures
-from typing import List, Optional, Union, Any, Dict
+from typing import List, Optional, Union, Any, Dict, Iterable
 
 import botocore.exceptions
 from botocore.model import OperationModel, Shape, StringShape, ListShape, StructureShape
@@ -25,6 +25,7 @@ from resotolib.baseresources import Cloud
 from resotolib.config import Config, RunningConfig, current_config
 from resotolib.core.actions import CoreFeedback
 from resotolib.core.custom_command import execute_command_on_resource
+from resotolib.core.progress import ProgressTree, ProgressDone
 from resotolib.graph import Graph
 from resotolib.logger import log, setup_logger
 from resotolib.types import JsonElement
@@ -66,13 +67,18 @@ class AWSCollectorPlugin(BaseCollectorPlugin):
         if len(accounts) == 0:
             log.error("No accounts found")
             return
+
+        progress = ProgressTree(self.cloud)
         for account in accounts:
+            # Even if the account is collected later, mark it as expected progress
+            progress.add_progress(ProgressDone(account.dname, 0, 1))
             add_str = ""
             if account.role:
                 add_str += f" role {account.role}"
             if account.profile:
                 add_str += f" profile {account.profile}"
             log.debug(f"Found {account.rtdname}{add_str}")
+        self.core_feedback.progress(progress)
 
         max_workers = len(accounts) if len(accounts) < Config.aws.account_pool_size else Config.aws.account_pool_size
         pool_args = {"max_workers": max_workers}
@@ -175,6 +181,19 @@ class AWSCollectorPlugin(BaseCollectorPlugin):
                 # map and structure types are currently not supported
                 raise ValueError(f"Cannot convert {o} to {shape}")
 
+        # it is not always possible to translate the operation or parameter name directly
+        # Example: modify-db-cluster --> ModifyDBCluster
+        #          --db-cluster-identifier --> --DBClusterIdentifier
+        # Use a lookup list for possible candidates
+        def get_name(is_name: str, possible_names: Iterable[str]) -> Optional[str]:
+            if is_name in possible_names:
+                return is_name
+            else:
+                for name in possible_names:
+                    if name.lower() == is_name.lower():
+                        return name
+                return None
+
         def coerce_args(fn_args: List[str], om: OperationModel) -> Dict[str, Any]:
             members: Dict[str, Shape] = om.input_shape.members if isinstance(om.input_shape, StructureShape) else {}
             param_name: Optional[str] = None
@@ -183,12 +202,13 @@ class AWSCollectorPlugin(BaseCollectorPlugin):
             for arg in fn_args:
                 if arg.startswith("--"):
                     name = pascalcase(arg.removeprefix("--"))
-                    param_name = name
-                    param_shape = members.get(name)
+                    param_name = get_name(name, members)
+                    if param_name is not None:
+                        param_shape = members.get(param_name)
                     bool_value = True
                     if param_shape is None and arg.startswith("--no-"):
                         name = name[2:]
-                        param_name = name
+                        param_name = get_name(name, members)
                         param_shape = members.get(name)
                         bool_value = False
                     if param_shape is None:
@@ -217,7 +237,8 @@ class AWSCollectorPlugin(BaseCollectorPlugin):
 
         # try to get the output shape of the operation
         service_model = client.service_model(p.service)
-        op: OperationModel = service_model.operation_model(pascalcase(p.operation))
+        op_name = get_name(pascalcase(p.operation), service_model.operation_names)
+        op: OperationModel = service_model.operation_model(op_name)
         output_shape = op.output_shape.type_name
         func_args = coerce_args(remaining, op)
 
@@ -312,7 +333,6 @@ class AWSCollectorPlugin(BaseCollectorPlugin):
     @staticmethod
     def cleanup(config: Config, resource: BaseResource, graph: Graph) -> bool:
         if isinstance(resource, AwsResource):
-
             client = get_client(config, resource)
 
             if resource.phantom:
@@ -536,7 +556,7 @@ def collect_account(
         Config.running_config.apply(running_config)
 
     if not authenticated(account, feedback):
-        log.error(f"Skipping account {account.rtdname}. Reason: authentication failure.")
+        feedback.error(f"Skipping account {account.rtdname}. Reason: authentication failure.", log)
         return None
 
     log.debug(f"Starting new collect process for account {account.dname}")
