@@ -20,7 +20,19 @@ from resotolib.utils import utc_str, log_runtime
 
 log = logging.getLogger("resoto.plugins.aws")
 
-RetryableErrors = ("RequestLimitExceeded", "Throttling", "TooManyRequestsException")
+RetryableErrors = {
+    "EC2ThrottledException",
+    "LimitExceededException",
+    "RequestLimitExceeded",
+    "RequestThrottled",
+    "RequestThrottledException",
+    "RequestTimeout",
+    "RequestTimeoutException",
+    "ThrottledException",
+    "Throttling",
+    "ThrottlingException",
+    "TooManyRequestsException",
+}
 T = TypeVar("T")
 
 
@@ -120,6 +132,8 @@ class AwsClient:
             return {key: self.__to_json(value, **kwargs) for key, value in node.items()}
         elif isinstance(node, datetime):
             return utc_str(node)
+        elif isinstance(node, bytes):
+            return node.decode("utf-8")
         else:
             raise AttributeError(f"Unsupported type: {type(node)}")
 
@@ -147,6 +161,9 @@ class AwsClient:
         except EndpointConnectionError as e:
             log.debug(f"The Aws endpoint does not exist in this region. Skipping. {e}")
             return None
+        except Exception as e:
+            log.error(f"[Aws] called service={aws_service} with resource: hit unexpected error: {e}")
+            raise
 
     def call_single(
         self, aws_service: str, action: str, result_name: Optional[str] = None, max_attempts: int = 1, **kwargs: Any
@@ -207,23 +224,35 @@ class AwsClient:
         except EndpointConnectionError as e:
             log.debug(f"The Aws endpoint does not exist in this region. Skipping. {e}")
             return None
+        except Exception as e:
+            log.error(f"[Aws] called service={aws_service} action={action}: hit unexpected error: {e}")
+            raise
 
     @log_runtime
     def call(
         self,
         aws_service: str,
         action: str,
-        result_name: Optional[str],
+        result_name: Optional[str] = None,
         expected_errors: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> JsonElement:
-        return self.call_single(aws_service, action, result_name, max_attempts=1, **kwargs)
+        try:
+            return self.call_single(aws_service, action, result_name, max_attempts=1, **kwargs)
+        except ClientError as e:
+            expected_errors = expected_errors or []
+            code = e.response["Error"]["Code"] or "Unknown Code"
+            if code in expected_errors:
+                log.debug(f"Expected error: {code}")
+                return None
+            else:
+                raise
 
     def list(
         self,
         aws_service: str,
         action: str,
-        result_name: Optional[str],
+        result_name: Optional[str] = None,
         expected_errors: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> List[Any]:
@@ -239,7 +268,7 @@ class AwsClient:
         self,
         aws_service: str,
         action: str,
-        result_name: Optional[str],
+        result_name: Optional[str] = None,
         expected_errors: Optional[List[str]] = None,
         **kwargs: Any,
     ) -> Optional[Json]:
