@@ -7,7 +7,7 @@ from concurrent.futures import Executor, Future
 from datetime import datetime, timezone
 from functools import lru_cache
 from threading import Lock
-from typing import ClassVar, Dict, Optional, List, Type, Any, TypeVar, Callable
+from typing import ClassVar, Dict, Optional, List, Type, Any, TypeVar, Callable, Iterator
 
 from attr import evolve, field
 from attrs import define
@@ -28,7 +28,7 @@ from resotolib.baseresources import (
 )
 from resotolib.config import Config, current_config
 from resotolib.core.actions import CoreFeedback
-from resotolib.graph import Graph
+from resotolib.graph import Graph, EdgeKey, ByNodeId, BySearchCriteria, NodeSelector
 from resotolib.json import to_json as to_js, from_json as from_js
 from resotolib.json_bender import Bender, bend
 from resotolib.types import Json
@@ -414,11 +414,26 @@ class GraphBuilder:
                 return n  # type: ignore
         return None
 
-    def add_node(self, node: AwsResourceType, source: Optional[Json] = None) -> AwsResourceType:
+    def nodes(self, clazz: Optional[Type[AwsResourceType]] = None, **node: Any) -> Iterator[AwsResourceType]:
+        for n in self.graph:
+            is_clazz = isinstance(n, clazz) if clazz else True
+            if is_clazz and all(getattr(n, k, None) == v for k, v in node.items()):
+                yield n
+
+    def add_node(
+        self, node: AwsResourceType, source: Optional[Json] = None, region: Optional[AwsRegion] = None
+    ) -> AwsResourceType:
+        """
+        Add a node to the graph.
+        :param node: the node to add.
+        :param source: the source json data.
+        :param region: only define the region in case it is different from the region of the graph builder.
+        :return: the added node
+        """
         log.debug(f"{self.name}: add node {node}")
         node._cloud = self.cloud
         node._account = self.account
-        node._region = self.region
+        node._region = region or self.region
         self.graph.add_node(node, source=source or {})
         return node
 
@@ -431,6 +446,14 @@ class GraphBuilder:
             log.debug(f"{self.name}: add edge: {start} -> {end} [{edge_type}]")
             self.graph.add_edge(start, end, edge_type=edge_type)
 
+    def add_deferred_edge(
+        self, from_node: BaseResource, edge_type: EdgeType, to_node: str, reverse: bool = False
+    ) -> None:
+        node1: NodeSelector = ByNodeId(from_node.chksum)
+        node2: NodeSelector = BySearchCriteria(to_node)
+        start, end = (node2, node1) if reverse else (node1, node2)
+        self.graph.add_deferred_edge(start, end, edge_type)
+
     def dependant_node(
         self, from_node: BaseResource, reverse: bool = False, delete_same_as_default: bool = False, **to_node: Any
     ) -> None:
@@ -447,15 +470,24 @@ class GraphBuilder:
     def resources_of(self, resource_type: Type[AwsResourceType]) -> List[AwsResourceType]:
         return [n for n in self.graph.nodes if isinstance(n, resource_type)]
 
+    def edges_of(
+        self, from_type: Type[AwsResource], to_type: Type[AwsResource], edge_type: EdgeType = EdgeType.default
+    ) -> List[EdgeKey]:
+        return [
+            key
+            for (from_node, to_node, key) in self.graph.edges
+            if isinstance(from_node, from_type) and isinstance(to_node, to_type) and key.edge_type == edge_type
+        ]
+
     @lru_cache(maxsize=None)
     def instance_type(self, region: AwsRegion, instance_type: str) -> Optional[Any]:
         if (it := self.global_instance_types.get(instance_type)) is None:
             return None  # instance type not found
 
         price = AwsPricingPrice.instance_type_price(self.client.for_region(region.id), instance_type, region.safe_name)
-        result = evolve(it, region=self.region, ondemand_cost=price.on_demand_price_usd if price else None)
+        result = evolve(it, region=region, ondemand_cost=price.on_demand_price_usd if price else None)
         # add this instance type to the graph
-        self.add_node(result)
+        self.add_node(result, region=region)
         self.add_edge(region, node=result)
         return result
 
