@@ -1,12 +1,14 @@
-from datetime import datetime
-from typing import ClassVar, Dict, Optional, Type, List, Any
+import csv
+import time
+from datetime import datetime, timedelta
+from typing import ClassVar, Dict, Optional, Type, List, Any, Callable
 
 from attrs import define, field
 
+from resoto_plugin_aws.aws_client import AwsClient
 from resoto_plugin_aws.resource.base import AwsResource, GraphBuilder, AwsApiSpec
 from resoto_plugin_aws.resource.ec2 import AwsEc2IamInstanceProfile
 from resoto_plugin_aws.utils import ToDict
-
 from resotolib.baseresources import (
     BaseCertificate,
     BasePolicy,
@@ -17,11 +19,11 @@ from resotolib.baseresources import (
     EdgeType,
     ModelReference,
 )
-from resotolib.json import from_json
+from resotolib.graph import Graph
+from resotolib.json import from_json, value_in_path
 from resotolib.json_bender import Bender, S, Bend, AsDate, Sort, bend, ForallBend, F
 from resotolib.types import Json
-from resotolib.graph import Graph
-from resoto_plugin_aws.aws_client import AwsClient
+from resotolib.utils import parse_utc, utc
 
 
 def iam_update_tag(resource: AwsResource, client: AwsClient, action: str, key: str, value: str, **kwargs: Any) -> bool:
@@ -72,8 +74,8 @@ class AwsIamAttachedPermissionsBoundary:
 @define(eq=False, slots=False)
 class AwsIamRoleLastUsed:
     kind: ClassVar[str] = "aws_iam_role_last_used"
-    mapping: ClassVar[Dict[str, Bender]] = {"last_used_date": S("LastUsedDate"), "region": S("Region")}
-    last_used_date: Optional[datetime] = field(default=None)
+    mapping: ClassVar[Dict[str, Bender]] = {"last_used": S("LastUsedDate"), "region": S("Region")}
+    last_used: Optional[datetime] = field(default=None)
     region: Optional[str] = field(default=None)
 
 
@@ -140,7 +142,6 @@ class AwsIamRole(AwsResource):
         )
 
     def pre_delete_resource(self, client: AwsClient, graph: Graph) -> bool:
-
         for successor in self.successors(graph, edge_type=EdgeType.delete):
             if isinstance(successor, AwsIamPolicy):
                 log_msg = f"Detaching {successor.rtdname}"
@@ -345,7 +346,6 @@ class AwsIamGroup(AwsResource, BaseGroup):
             builder.dependant_node(self, clazz=AwsIamPolicy, delete_same_as_default=True, arn=policy.get("PolicyArn"))
 
     def pre_delete_resource(self, client: AwsClient, graph: Graph) -> bool:
-
         for successor in self.successors(graph, edge_type=EdgeType.delete):
             if isinstance(successor, AwsIamPolicy):
                 log_msg = f"Detaching {successor.rtdname}"
@@ -393,11 +393,13 @@ class AwsIamGroup(AwsResource, BaseGroup):
 class AwsIamAccessKeyLastUsed:
     kind: ClassVar[str] = "aws_iam_access_key_last_used"
     mapping: ClassVar[Dict[str, Bender]] = {
-        "last_used_date": S("LastUsedDate"),
+        "last_used": S("LastUsedDate"),
+        "last_rotated": S("LastRotated"),
         "service_name": S("ServiceName"),
         "region": S("Region"),
     }
-    last_used_date: Optional[datetime] = field(default=None)
+    last_used: Optional[datetime] = field(default=None)
+    last_rotated: Optional[datetime] = field(default=None)
     service_name: Optional[str] = field(default=None)
     region: Optional[str] = field(default=None)
 
@@ -421,6 +423,123 @@ class AwsIamAccessKey(AwsResource, BaseAccessKey):
     access_key_last_used: Optional[AwsIamAccessKeyLastUsed] = field(default=None)
 
 
+class CredentialReportLine:
+    undefined = {"not_supported", "N/A"}
+
+    def __init__(self, line: Dict[str, str]) -> None:
+        self.line = line
+
+    def add_root_user(self, builder: GraphBuilder) -> None:
+        user = AwsRootUser(
+            id="root",
+            name="root",
+            arn=self.value_of("arn"),
+            ctime=self.value_of("user_creation_time", parse_utc),
+            password_enabled=self.password_enabled(),
+            password_last_used=self.password_last_used(),
+            password_last_changed=self.password_last_changed(),
+            password_next_rotation=self.password_next_rotation(),
+            mfa_active=self.mfa_active(),
+        )
+        builder.add_node(user)
+        for key in self.access_keys():
+            if key.access_key_status == "Active" or key.access_key_last_used is not None:
+                builder.add_node(key)
+                builder.add_edge(user, node=key)
+
+    def access_keys(self) -> List[AwsIamAccessKey]:
+        def by_index(i: int) -> AwsIamAccessKey:
+            last_used = self.value_of(f"access_key_{i}_last_used_date", parse_utc)
+            service_name = self.value_of(f"access_key_{i}_last_used_service")
+            region = self.value_of(f"access_key_{i}_last_used_region")
+            last_rotated = self.value_of(f"access_key_{i}_last_rotated", parse_utc)
+            last_used = (
+                None
+                if last_used is None and service_name is None and region is None and last_rotated is None
+                else AwsIamAccessKeyLastUsed(
+                    last_used=last_used, last_rotated=last_rotated, service_name=service_name, region=region
+                )
+            )
+            return AwsIamAccessKey(
+                id=f"root_key_{i}",
+                name=f"root_key_{i}",
+                access_key_status="Active" if self.value_of(f"access_key_{i}_active") == "true" else "Inactive",
+                atime=last_used.last_used if last_used else None,
+                access_key_last_used=last_used,
+            )
+
+        # the report holds 2 entries
+        return [by_index(idx) for idx in range(1, 3)]
+
+    def value_of(self, k: str, fn: Optional[Callable[[str], Any]] = None) -> Any:
+        v = self.line.get(k)
+        return None if v is None or v in self.undefined else (fn(v) if fn else v)
+
+    def password_enabled(self) -> bool:
+        return self.value_of("password_enabled") == "true"  # type: ignore
+
+    def password_last_used(self) -> Optional[datetime]:
+        return self.value_of("password_last_used", parse_utc)  # type: ignore
+
+    def password_last_changed(self) -> Optional[datetime]:
+        return self.value_of("password_last_changed", parse_utc)  # type: ignore
+
+    def password_next_rotation(self) -> Optional[datetime]:
+        return self.value_of("password_next_rotation", parse_utc)  # type: ignore
+
+    def mfa_active(self) -> bool:
+        return self.value_of("mfa_active") == "true"  # type: ignore
+
+    @staticmethod
+    def user_lines(builder: GraphBuilder) -> Dict[str, "CredentialReportLine"]:
+        started_at = utc()
+        # wait for the report to be done
+        while (
+            # in case of access denied, res will be None
+            (res := builder.client.get("iam", "generate-credential-report"))
+            # res is defined, but the report is not ready yet
+            and res.get("State") != "COMPLETE"
+            # give up after 5 minutes
+            and (utc() - started_at) < timedelta(minutes=5)
+        ):
+            time.sleep(1)
+        # fetch the report
+        if res and res.get("State") == "COMPLETE":
+            report = builder.client.get("iam", "get-credential-report", expected_errors=["ReportNotPresent"])
+            return CredentialReportLine.from_str(report["Content"]) if report else {}
+        else:
+            return {}
+
+    @staticmethod
+    def from_str(lines: str) -> Dict[str, "CredentialReportLine"]:
+        return {i["user"]: CredentialReportLine(i) for i in csv.DictReader(lines.splitlines(), delimiter=",")}
+
+
+@define(eq=False, slots=False)
+class AwsIamVirtualMfaDevice:
+    kind: ClassVar[str] = "aws_iam_virtual_mfa_device"
+    mapping: ClassVar[Dict[str, Bender]] = {
+        "serial_number": S("SerialNumber"),
+        "enable_date": S("EnableDate"),
+    }
+    serial_number: Optional[str] = field(default=None)
+    enable_date: Optional[datetime] = field(default=None)
+
+
+@define(eq=False, slots=False)
+class AwsRootUser(AwsResource, BaseUser):
+    kind: ClassVar[str] = "aws_root_user"
+    reference_kinds: ClassVar[ModelReference] = {
+        "predecessors": {"default": ["aws_account"]},
+    }
+    password_enabled: Optional[bool] = field(default=None)
+    password_last_used: Optional[datetime] = field(default=None)
+    password_last_changed: Optional[datetime] = field(default=None)
+    password_next_rotation: Optional[datetime] = field(default=None)
+    mfa_active: Optional[bool] = field(default=None)
+    user_virtual_mfa_devices: Optional[List[AwsIamVirtualMfaDevice]] = field(default=None)
+
+
 @define(eq=False, slots=False)
 class AwsIamUser(AwsResource, BaseUser):
     kind: ClassVar[str] = "aws_iam_user"
@@ -434,7 +553,6 @@ class AwsIamUser(AwsResource, BaseUser):
         "tags": S("Tags", default=[]) >> ToDict(),
         "name": S("UserName"),
         "ctime": S("CreateDate"),
-        "atime": S("PasswordLastUsed"),
         "path": S("Path"),
         "arn": S("Arn"),
         "user_policies": S("UserPolicyList", default=[]) >> ForallBend(AwsIamPolicyDetail.mapping),
@@ -443,6 +561,12 @@ class AwsIamUser(AwsResource, BaseUser):
     path: Optional[str] = field(default=None)
     user_policies: List[AwsIamPolicyDetail] = field(factory=list)
     user_permissions_boundary: Optional[AwsIamAttachedPermissionsBoundary] = field(default=None)
+    password_enabled: Optional[bool] = field(default=None)
+    password_last_used: Optional[datetime] = field(default=None)
+    password_last_changed: Optional[datetime] = field(default=None)
+    password_next_rotation: Optional[datetime] = field(default=None)
+    mfa_active: Optional[bool] = field(default=None)
+    user_virtual_mfa_devices: Optional[List[AwsIamVirtualMfaDevice]] = field(default=None)
 
     @classmethod
     def called_collect_apis(cls) -> List[AwsApiSpec]:
@@ -450,15 +574,26 @@ class AwsIamUser(AwsResource, BaseUser):
             cls.api_spec,
             AwsApiSpec("iam", "list-access-keys"),
             AwsApiSpec("iam", "get-access-key-last-used"),
-            AwsApiSpec("iam", "list-users"),
+            AwsApiSpec("iam", "generate-credential-report"),
+            AwsApiSpec("iam", "get-credential-report"),
         ]
 
+    @classmethod
+    def collect_resources(cls: Type[AwsResource], builder: GraphBuilder) -> None:
+        # start generation of the credentials resport and pick it up later
+        builder.client.get("iam", "generate-credential-report")
+        # let super handle the rest (this will take some time for the report to be done)
+        super().collect_resources(builder)  # type: ignore # mypy bug: https://github.com/python/mypy/issues/12885
+
     @classmethod
     def collect(cls: Type[AwsResource], json_list: List[Json], builder: GraphBuilder) -> None:
-        name_password_last_used_map: Dict[str, str] = {}
-        for user in builder.client.list("iam", "list-users", "Users"):
-            if "PasswordLastUsed" in user and "UserId" in user:
-                name_password_last_used_map[user["UserId"]] = user["PasswordLastUsed"]
+        # retrieve the created report
+        report = CredentialReportLine.user_lines(builder)
+
+        # the root user is not listed in IAM users, so we need to add it manually
+        if root_user := report.get("<root_account>"):
+            root_user.add_root_user(builder)
+
         for json in json_list:
             for js in json.get("GroupDetailList", []):
                 builder.add_node(AwsIamGroup.from_api(js), js)
@@ -470,21 +605,41 @@ class AwsIamUser(AwsResource, BaseUser):
                 builder.add_node(AwsIamPolicy.from_api(js), js)
 
             for js in json.get("UserDetailList", []):
-                js["PasswordLastUsed"] = name_password_last_used_map.get(js["UserId"])
                 user = AwsIamUser.from_api(js)
                 builder.add_node(user, js)
+                line = report.get(user.name or user.id)
+                line_keys: List[AwsIamAccessKey] = []
+                if line:
+                    user.password_enabled = line.password_enabled()
+                    user.password_last_used = line.password_last_used()
+                    user.atime = user.password_last_used
+                    user.password_last_changed = line.password_last_changed()
+                    user.password_next_rotation = line.password_next_rotation()
+                    user.mfa_active = line.mfa_active()
+                    line_keys = line.access_keys()
                 # add all iam access keys for this user
-                for ak in builder.client.list("iam", "list-access-keys", "AccessKeyMetadata", UserName=user.name):
+                for idx, ak in enumerate(
+                    builder.client.list("iam", "list-access-keys", "AccessKeyMetadata", UserName=user.name)
+                ):
                     key = AwsIamAccessKey.from_api(ak)
-                    # get last used date for this key
-                    if lu := builder.client.get(
-                        "iam", "get-access-key-last-used", "AccessKeyLastUsed", AccessKeyId=key.id
-                    ):
-                        key.access_key_last_used = AwsIamAccessKeyLastUsed.from_api(lu)
-                        key.atime = key.access_key_last_used.last_used_date if key.access_key_last_used else None
+                    if line and idx < len(line_keys):
+                        key.access_key_last_used = line_keys[idx].access_key_last_used
                     builder.add_node(key, ak)
                     builder.dependant_node(user, node=key)
 
+        def add_virtual_mfa_devices() -> None:
+            for vjs in builder.client.list("iam", "list-virtual-mfa-devices", "VirtualMFADevices"):
+                if arn := value_in_path(vjs, "User.Arn"):
+                    if isinstance(usr := builder.node(arn=arn), (AwsIamUser, AwsRootUser)):
+                        mapped = bend(AwsIamVirtualMfaDevice.mapping, vjs)
+                        node = from_json(mapped, AwsIamVirtualMfaDevice)
+                        if usr.user_virtual_mfa_devices is None:
+                            usr.user_virtual_mfa_devices = []
+                        usr.user_virtual_mfa_devices.append(node)
+
+        if builder.account.mfa_devices is not None and builder.account.mfa_devices > 0:
+            builder.submit_work(add_virtual_mfa_devices)
+
     def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
         for p in bend(S("AttachedManagedPolicies", default=[]), source):
             builder.dependant_node(self, clazz=AwsIamPolicy, delete_same_as_default=True, arn=p.get("PolicyArn"))
