@@ -3,6 +3,8 @@ from typing import ClassVar, Dict, Optional, List
 from attrs import define, field
 
 from resoto_plugin_aws.resource.base import AwsResource, GraphBuilder, AwsApiSpec
+from resoto_plugin_aws.resource.kms import AwsKmsKey
+from resotolib.baseresources import ModelReference
 from resotolib.json_bender import Bender, S, Bend, bend, ForallBend
 from resotolib.types import Json
 from resoto_plugin_aws.aws_client import AwsClient
@@ -59,6 +61,12 @@ class AwsKinesisEnhancedMetrics:
 @define(eq=False, slots=False)
 class AwsKinesisStream(AwsResource):
     kind: ClassVar[str] = "aws_kinesis_stream"
+    reference_kinds: ClassVar[ModelReference] = {
+        "predecessors": {
+            "delete": ["aws_kms_key"],
+        },
+        "successors": {"default": ["aws_kms_key"]},
+    }
     api_spec: ClassVar[AwsApiSpec] = AwsApiSpec("kinesis", "list-streams", "StreamNames")
     mapping: ClassVar[Dict[str, Bender]] = {
         "id": S("StreamName"),
@@ -112,6 +120,10 @@ class AwsKinesisStream(AwsResource):
                 builder.add_node(stream)
                 builder.submit_work(add_tags, stream)
 
+    def connect_in_graph(self, builder: GraphBuilder, source: Json) -> None:
+        if self.kinesis_key_id:
+            builder.dependant_node(self, clazz=AwsKmsKey, id=AwsKmsKey.normalise_id(self.kinesis_key_id))
+
     def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
         client.call(
             aws_service=self.api_spec.service,
