@@ -1,24 +1,21 @@
 from typing import ClassVar, Dict, Optional, List, Type, cast
 
 from attrs import define, field
+from resoto_plugin_aws.aws_client import AwsClient
 
 from resoto_plugin_aws.resource.base import AwsResource, AwsApiSpec, GraphBuilder
+from resoto_plugin_aws.utils import ToDict
 
-from resotolib.baseresources import (  # noqa: F401
-    BaseCertificate,
-    BasePolicy,
-    BaseGroup,
-    BaseAccount,
-    BaseAccessKey,
-    BaseUser,
+from resotolib.baseresources import (
     BaseDNSZone,
     BaseDNSRecord,
     EdgeType,
     BaseDNSRecordSet,
     ModelReference,
 )
-from resotolib.json_bender import Bender, S, Bend, ForallBend
+from resotolib.json_bender import F, Bender, S, Bend, ForallBend, bend
 from resotolib.types import Json
+from resotolib.utils import rrdata_as_dict
 
 
 @define(eq=False, slots=False)
@@ -48,7 +45,7 @@ class AwsRoute53Zone(AwsResource, BaseDNSZone):
         }
     }
     mapping: ClassVar[Dict[str, Bender]] = {
-        "id": S("Id"),
+        "id": S("Id") >> F(lambda x: x.rsplit("/", 1)[-1]),  # remove leading "/hostedzones/"
         "name": S("Name"),
         "zone_caller_reference": S("CallerReference"),
         "zone_config": S("Config") >> Bend(AwsRoute53ZoneConfig.mapping),
@@ -60,11 +57,31 @@ class AwsRoute53Zone(AwsResource, BaseDNSZone):
     zone_resource_record_set_count: Optional[int] = field(default=None)
     zone_linked_service: Optional[AwsRoute53LinkedService] = field(default=None)
 
+    @classmethod
+    def called_collect_apis(cls) -> List[AwsApiSpec]:
+        return [
+            cls.api_spec,
+            AwsApiSpec("route53", "list-resource-record-sets"),
+            AwsApiSpec("route53", "list-tags-for-resource"),
+        ]
+
     @classmethod
     def collect(cls: Type[AwsResource], json: List[Json], builder: GraphBuilder) -> None:
+        def add_tags(zone: AwsRoute53Zone) -> None:
+            tags = builder.client.get(
+                "route53",
+                "list-tags-for-resource",
+                result_name="ResourceTagSet",
+                ResourceType="hostedzone",
+                ResourceId=zone.id,
+            )
+            if tags:
+                zone.tags = bend(S("Tags", default=[]) >> ToDict(), tags)
+
         for js in json:
             zone: AwsRoute53Zone = cast(AwsRoute53Zone, cls.from_api(js))
             builder.add_node(zone, js)
+            builder.submit_work(add_tags, zone)
             for rs_js in builder.client.list(
                 "route53", "list-resource-record-sets", "ResourceRecordSets", HostedZoneId=zone.id
             ):
@@ -78,12 +95,45 @@ class AwsRoute53Zone(AwsResource, BaseDNSZone):
                         record_type=record_set.record_type,
                         record_ttl=record_set.record_ttl or 0,
                         record_data=data,
-                        record_value="foo",
+                        **rrdata_as_dict(record_set.record_type, data),
                     )
                     builder.add_node(record, js)
                     builder.add_edge(record_set, EdgeType.default, node=record)
                     builder.add_edge(record_set, EdgeType.delete, node=record)
 
+    def update_resource_tag(self, client: AwsClient, key: str, value: str) -> bool:
+        client.call(
+            aws_service="route53",
+            action="change-tags-for-resource",
+            result_name=None,
+            ResourceType="hostedzone",
+            ResourceId=self.id,
+            AddTags=[{"Key": key, "Value": value}],
+        )
+        return True
+
+    def delete_resource_tag(self, client: AwsClient, key: str) -> bool:
+        client.call(
+            aws_service="route53",
+            action="change-tags-for-resource",
+            result_name=None,
+            ResourceType="hostedzone",
+            ResourceId=self.id,
+            RemoveTagKeys=[key],
+        )
+        return True
+
+    def delete_resource(self, client: AwsClient) -> bool:
+        client.call(aws_service="route53", action="delete-hosted-zone", result_name=None, Id=self.id.rsplit("/", 1)[-1])
+        return True
+
+    @classmethod
+    def called_mutator_apis(cls) -> List[AwsApiSpec]:
+        return [
+            AwsApiSpec("route53", "change-tags-for-resource"),
+            AwsApiSpec("route53", "delete-hosted-zone"),
+        ]
+
 
 @define(eq=False, slots=False)
 class AwsRoute53GeoLocation:
@@ -168,5 +218,21 @@ class AwsRoute53ResourceRecordSet(AwsResource, BaseDNSRecordSet):
     record_traffic_policy_instance_id: Optional[str] = field(default=None)
     record_cidr_routing_config: Optional[AwsRoute53CidrRoutingConfig] = field(default=None)
 
+    def _keys(self) -> tuple[str, str, str, str, str, str, str, str, str, Optional[str]]:
+        if self._graph is None:
+            raise RuntimeError(f"_keys() called on {self.rtdname} before resource was added to graph")
+        return (
+            self.kind,
+            self.cloud().id,
+            self.account().id,
+            self.region().id,
+            self.zone().id,
+            self.dns_zone().id,
+            self.id,
+            self.safe_name,
+            self.record_type,
+            self.record_set_identifier,
+        )
+
 
 resources: List[Type[AwsResource]] = [AwsRoute53Zone, AwsRoute53ResourceRecord, AwsRoute53ResourceRecordSet]
