@@ -2,24 +2,29 @@ from test.resources import round_trip_for
 from types import SimpleNamespace
 from typing import cast, Any
 from resoto_plugin_aws.aws_client import AwsClient
-from resoto_plugin_aws.resource.rds import AwsRdsInstance
+from resoto_plugin_aws.resource.rds import AwsRdsInstance, AwsRdsCluster
 
 
 def test_rds_instances() -> None:
     first, builder = round_trip_for(AwsRdsInstance)
     assert len(builder.resources_of(AwsRdsInstance)) == 2
+    assert len(first.tags) == 1
+
+
+def test_rds_cluster() -> None:
+    round_trip_for(AwsRdsCluster)
 
 
 def test_tagging() -> None:
     instance, _ = round_trip_for(AwsRdsInstance)
 
     def validate_update_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "add_tags_to_resource"
+        assert kwargs["action"] == "add-tags-to-resource"
         assert kwargs["ResourceName"] == instance.arn
         assert kwargs["Tags"] == [{"Key": "foo", "Value": "bar"}]
 
     def validate_delete_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "remove_tags_from_resource"
+        assert kwargs["action"] == "remove-tags-from-resource"
         assert kwargs["ResourceName"] == instance.arn
         assert kwargs["TagKeys"] == ["foo"]
 
@@ -34,7 +39,7 @@ def test_deletion() -> None:
     instance, _ = round_trip_for(AwsRdsInstance)
 
     def validate_delete_args(**kwargs: Any) -> None:
-        assert kwargs["action"] == "delete_db_instance"
+        assert kwargs["action"] == "delete-db-instance"
         assert kwargs["DBInstanceIdentifier"] == instance.name
         assert kwargs["SkipFinalSnapshot"] is True
 
